#include "ppc.h"
#include "M33.h"

PPC::PPC(float hfov, int _w, int _h) {

	w = _w;
	h = _h;
	a = V3(1.0f, 0.0f, 0.0f);
	b = V3(0.0f, -1.0f, 0.0f);
	C = V3(0.0f, 0.0f, 0.0f);
	float hfovRad = hfov * 3.14159f / 180.0f;
	c = V3(-(float)w / 2.0f, (float)h / 2.0f, -(float)w/(2.0f*tanf(hfovRad/2.0f)));

}

int PPC::Project(V3 P, V3 &pP) {
	// 0 point behind head
	// 1 point in front of head

	M33 M;
	M.SetColumn(0, a);
	M.SetColumn(1, b);
	M.SetColumn(2, c);

	M33 Minv = M.Inverted();
	M33 tmp = M * Minv;
	V3 Q = Minv * (P - C);
	if (Q[2] <= 0.0f)
		return 0;

	pP = V3(Q[0] / Q[2], Q[1] / Q[2], 1.0f / Q[2]);
	return 1;

}



void PPC::Pan(float theta) {

	V3 aD = (b * -1.0f);
	a = a.RotateThisVectorAboutAxis(aD, theta);
	c = c.RotateThisVectorAboutAxis(aD, theta);

}

void PPC::Tilt(float theta) {

	V3 aD = a;
	b = b.RotateThisVectorAboutAxis(aD, theta);
	c = c.RotateThisVectorAboutAxis(aD, theta);

}

void PPC::Roll(float theta) {

	V3 aD = a^b;
	a = a.RotateThisVectorAboutAxis(aD, theta);
	b = b.RotateThisVectorAboutAxis(aD, theta);
	c = c.RotateThisVectorAboutAxis(aD, theta);

}




