#include "scene.h"

#include "V3.h"
#include "M33.h"
#include "ppc.h"
#include "tm.h"

Scene *scene;

using namespace std;

#include <iostream>
#include <fstream>

Scene::Scene() {


	gui = new GUI();
	gui->show();

	int u0 = 16;
	int v0 = 40;
	int h = 400;
	int w = 600;
	fb = new FrameBuffer(u0, v0, w, h);
	fb->position(u0, v0);
	fb->label("Framebuffer");
	fb->show();
	fb->SetBGR(0xFF0000FF);
	fb->redraw();
	gui->uiw->position(u0, v0 + fb->h + v0);

	float hfov = 55.0f;
	ppc = new PPC(hfov, fb->w, fb->h);
	tmsN = 1;
	tms = new TM[tmsN];
	tms[0].LoadBin("geometry/teapot1K.bin");
	V3 tv(20.0f, -20.0f, -200.0f);
	tms[0].Translate(tv);

}

void Scene::Render() {

	fb->SetBGR(0xFFFFFFFF);
	fb->ClearZB();
	int psize = 5;
	for (int tmi = 0; tmi < tmsN; tmi++) {
		tms[tmi].RenderPoints(fb, psize, ppc);
	}
	fb->redraw();
}

void Scene::DBG() {

	{
		int framesN = 720;
		V3 tmc = tms[0].GetCenter();
		for (int fi = 0; fi < framesN; fi++) {
//			tms[0].Translate(V3(0.1f, 0.0f, 0.0f));
			tms[0].Rotate(tmc, V3(0.0f, 1.0f, 0.0f), 1.0f);
//			ppc->Roll(1.f);
			Render();
			Fl::check();
		}
		return;
	}

	{
		float hfov = 55.0f;
		PPC ppc(hfov, fb->w, fb->h);

		TM tm;
		tm.LoadBin("geometry/teapot1K.bin");
		V3 tv(20.0f, -20.0f, -200.0f);
		tm.Translate(tv);

		fb->SetBGR(0xFFFFFFFF);
		fb->ClearZB();

		tm.RenderPoints(fb, 3, &ppc);
		fb->redraw();
		return;

		V3 q[4];
		float hw = 10.0f;
		float z = -40.0f;
		q[0] = V3(-hw, +hw, z);
		q[1] = V3(-hw, -hw, z);
		q[2] = V3(+hw, -hw, z);
		q[3] = V3(+hw, +hw, z);

		V3 pP;
		ppc.Project(q[0], pP);
		cerr << pP << endl;
		ppc.Project(q[1], pP);
		cerr << pP << endl;
		ppc.Project(q[2], pP);
		cerr << pP << endl;
		ppc.Project(q[3], pP);
		cerr << pP << endl;

		TM quad;
		quad.SetQuad(q[0], q[1], q[2], q[3]);

		fb->SetBGR(0xFFFFFFFF);
		for (int vi = 0; vi < quad.vertsN; vi++) {
			fb->Draw3DPoint(quad.verts[vi], V3(0.0f, 0.0f, 0.0f), 7, &ppc);
		}
		fb->redraw();
		return;
	}

	cerr << "INFO: pressed DBG" << endl;
}

void Scene::NewButton() {
	cerr << "INFO: pressed New button on GUI" << endl;
}




